#! /bin/sh

### Richscrips Installer ######################################################
#                                                                             #
#  Type the following command as root.                                        #
#  "sh install.sh"                                                            #
#                                                                             #
###############################################################################


# =============================================================================
# Make sure some conditions
# -----------------------------------------------------------------------------
#  1. I am root
#  2. executed in the same path
#  3. OS type
# =============================================================================
#
# --- 1. I am root -----
check=`whoami`
if [ "$check" != "root" ]; then
  echo "This script must be executed as root."
  exit 1
fi
# --- 2. executed in the same path -----
if [ \( "$0" != "${0##*/}" \) -a \( "${0##*/}" != "${0##./}" \) ]; then
 echo "Do not execute with absolute path or in a diffrent directory."
 echo "You may type \"sh ${0##*/}\" or \"./${0##*/}\"."
 exit 1
fi
# --- 3. OS type -----
os_type=`uname`


# =============================================================================
# Ask some questions
# -----------------------------------------------------------------------------
#  1. connection envioronment
#  2. inner network #2
#  3. dhclient.conf path
#  4. IIJ-base PPP configuration files path
#  5. making sure
# =============================================================================
#
# --- 1. connection environment -----
check=""
while [ -z "$connection_env" ]; do
  echo "Choose your connecting tool from the following."
  echo "  1. A fixed IP address has been already assigned"
  echo "  2. Using \"dhclient\" to assign an IP address"
  echo "  3. Using \"user-ppp\"(IIJ-PPP) to assign an IP address"
  echo "  4. Using \"mpd\" to assign an IP address (cannot use yet!)"
  echo "  :"
  echo "  0. other (no configuration files about this will be install.)"
  echo -n "Input (0-3) ? "
  read check
  case $check in
    0)
      connection_env="NONE"
      ;;
    1)
      connection_env="FIXED"
      ;;
    2)
      connection_env="DHCP"
      ;;
    3)
      connection_env="USER-PPP"
      ;;
    *)
      echo "Bad number!"
      ;;
   esac
done
# --- 2. inner network #2 -----
check=""
while [ -z "$inner_nw2" ]; do
  echo "Choose a number about the inner network #2."
  echo "The inner network #2 is required only when you want use OpenVPN or UZIX."
  echo "  0. No use (Choose this if you have no plan to use OpenVPN or UZIX.)"
  echo "  1. Use with \"user-ppp\"(IIJ-PPP)"
  echo -n "Input (0-1) ? "
  read check
  case $check in
    0)
      inner_nw2="NONE"
      ;;
    1)
      inner_nw2="USER-PPP"
      ;;
    *)
      echo "Bad number!"
      ;;
   esac
done
# --- 3. dhclient.conf path -----
if [ $connection_env = "DHCP" ]; then
  check=""
  while [ -z "$dhclient_path" ]; do
    echo "Tell me the directory path of \"dhclinet.conf\" you use now."
    echo -n "Input (absolute path) ? "
    read check
    if [ \( "$check" = "" \) -o \( "${check%%/*}" != "" \) ]; then
      echo "Specify with absolute path!"
      continue
    fi
    if [ "${check##*/}" = "dhclient.conf" ]; then
      check=${check%/dhclient.conf}
    fi
    check="${check%/}"
    if [ -f "$check/dhclient.conf" ]; then
      [ -z "$check" ] && check="/"
      dhclient_path="$check"
    else
      echo "Not found!"
    fi
  done
fi
# --- 4. user-ppp configuration files path -----
if [ \( $connection_env = "USER-PPP" \) -o \( $inner_nw2 = "USER-PPP" \) ]; then
  check=""
  while [ -z "$ppp_path" ]; do
    echo "Tell me the directory path of \"ppp\"(user-ppp) configuration files you use now."
    echo -n "Input (absolute path) ? "
    read check
    if [ \( "$check" = "" \) -o \( "${check%%/*}" != "" \) ]; then
      echo "Specify with absolute path!"
      continue
    fi
    check=${check%/}
    if [ "${check##*/}" != "ppp" ]; then
      echo "Not \"ppp\" directory!"
      continue
    fi
    check="${check%/ppp}"
    if [ -d "$check/ppp" ]; then
      [ -z "$check" ] && check="/"
      ppp_path="$check"
    else
      echo "Not found!"
    fi
  done
fi
# --- 5. making sure -----
echo -n "Push any key to begin to install. "
read check


# =============================================================================
# Install
# -----------------------------------------------------------------------------
#  0. definition
#  1. make temporary directory
#  2. install the files in "usr_local_richscripts"
#  3. install the files for "dhclient" or "user-ppp"
#  4. install some OS depending files
#  5. delete temporary directory
# =============================================================================
#
# ----- 0. definition -----
rename_as_samble() {
  local srcfile
  local dstfile
  srcfile="tmp/richscripts/etc/$1"
  dstfile="/usr/local/richscripts/etc/$1"
  if [ -f $dstfile ]; then
    mv $srcfile ${srcfile}.samble
  fi
}
# ----- 1. make temporary directory -----
mkdir tmp
# ----- 2. install the files in "usr_local_richscripts" -----
cp -R usr_local_richscripts tmp/richscripts
chmod 544 tmp/richscripts/bin/*.sh
chmod 644 tmp/richscripts/etc/*
rename_as_samble common_local.lib
rename_as_samble ddnsup.src
rename_as_samble ipfrule.src
rename_as_samble ipfwrule_dynamic.src
rename_as_samble ipfwrule_static.src
cp -R tmp/richscripts /usr/local
# ----- 3. install the files for "dhclient" or "user-ppp" -----
if [ $connection_env = "DHCP" ]; then
  cp dhclient/* tmp
  check=`grep "\-x /etc/dhclient-" $dhclient_path/../sbin/dhclient-script`
  if [ -n "$check" ]; then
    chmod u+x tmp/dhclient-*-hooks
  fi
  [ -f $dhclient_path/dhclient-enter-hooks ] && \
  mv $dhclient_path/dhclient-enter-hooks \
  $dhclient_path/dhclient-enter-hooks.bak
  [ -f $dhclient_path/dhclient-exit-hooks ] && \
  mv $dhclient_path/dhclient-exit-hooks \
  $dhclient_path/dhclient-exit-hooks.bak
  cp tmp/dhclient-*-hooks $dhclient_path
fi
if [ \( $connection_env = "USER-PPP" \) -o \( $inner_nw2 = "USER-PPP" \) ]; then
  cp -R ppp tmp
  chmod 644 tmp/ppp/*
  chmod go-r tmp/ppp/ppp.conf
  if [ -d $ppp_path ]; then
    [ -d $ppp_path/ppp.bak ] && rm -rf $ppp_path/ppp.bak
    mv $ppp_path/ppp $ppp_path/ppp.bak
  fi
  cp -R tmp/ppp $ppp_path
fi
# ----- 4. install OS depending files -----
if [ -d "$os_type" ]; then
  cp -R $os_type tmp
  case $os_type in
    FreeBSD)
      if [ \( $connection_env = "USER-PPP" \) -o \
           \( $inner_nw2 = "USER-PPP" \) ]; then
        chmod 644 $os_type/etc/*
        cp $os_type/etc/* /etc
      fi
      chmod 744 boot/*
      cp boot/boot_luwaiter.sh /usr/local/etc/rc.d/005.luwaiter.sh
      ;;
    Linux)
      chmod 744 boot/*
      cp boot/boot_luwaiter.sh /etc/init.d/luwaiter
      ;;
  esac
fi
# ----- 5. delete temporary directory -----
rm -rf tmp


# =============================================================================
# Create commands.lib
# -----------------------------------------------------------------------------
#  0. definition
#  1. create the file
#  2. write commands to the file
#  3. chmod
# =============================================================================
#
# ----- 0. definition -----
make_cmd_var() {
  local output
  local path_cmd
  [ \( -z "$1" \) -o \( -z "$2" \) ] && return 1
  path_cmd=`which $2`
  [ -n "${path_cmd%%/*}" ] && path_cmd=""
  output="${1}=\"$path_cmd\""
  echo $output >> $PATH_CMDPATHLIB
}
PATH_CMDPATHLIB="/usr/local/richscripts/etc/common_cmds.lib"
# ----- 1. create the file -----
echo -n > $PATH_CMDPATHLIB
# ----- 2. write commands to the file -----
echo "# ----- OS type -----" >> $PATH_CMDPATHLIB
echo "OS_TYPE=\"$os_type\"" >> $PATH_CMDPATHLIB
echo "" >> $PATH_CMDPATHLIB
echo "# ----- general command -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_CAT      cat
make_cmd_var CMD_DIFF     diff
make_cmd_var CMD_FIND     find
make_cmd_var CMD_GREP     grep
make_cmd_var CMD_IFCONFIG ifconfig
make_cmd_var CMD_KILL     kill
make_cmd_var CMD_MKDIR    mkdir
make_cmd_var CMD_MKTEMP   mktemp
make_cmd_var CMD_MV       mv
make_cmd_var CMD_PS       ps
make_cmd_var CMD_RM       rm
make_cmd_var CMD_RMDIR    rmdir
make_cmd_var CMD_ROUTE    route
make_cmd_var CMD_SLEEP    sleep
make_cmd_var CMD_SORT     sort
make_cmd_var CMD_TOUCH    touch
echo "" >> $PATH_CMDPATHLIB
echo "# ----- Firewall commands -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_IPFW     ipfw
make_cmd_var CMD_IPF      ipf
make_cmd_var CMD_IPNAT    ipnat
make_cmd_var CMD_IPCHAINS ipchains
make_cmd_var CMD_IPTABLES iptables
echo "" >> $PATH_CMDPATHLIB
echo "# ----- ppp -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_PPP      ppp
make_cmd_var CMD_MPD      mpd
echo "" >> $PATH_CMDPATHLIB
echo "# ----- daemontools -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_SVC      svc
echo "" >> $PATH_CMDPATHLIB
echo "# ----- FreeBSD commands -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_RTPRIO   rtprio
echo "# ----- OpenVPN command -----" >> $PATH_CMDPATHLIB
make_cmd_var CMD_OPENVPN  openvpn
# ----- 3. chmod -----
chmod 644 $PATH_CMDPATHLIB


# =============================================================================
# Install completed
# =============================================================================
#
echo "Richscripts has been installed."
echo "Customize the configuration files in /usr/local/richscripts/etc"
echo "and some OS depending files in /etc."
