#! /bin/sh

### mpdscrpt.sh ###############################################################
#                                                                             #
#  Usage :                                                                    #
#                                                                             #
#  You just write into the mpd.conf                                           #
#  as a "iface up-script" and "iface up-script" .                             #
#                                                                             #
###############################################################################


# =============================================================================
# Definition
# =============================================================================
#
# ----- define common variables and functions -----
. /usr/local/richscripts/etc/common.lib


# =============================================================================
# Parse the arguments
# -----------------------------------------------------------------------------
#  1. parse the IP addresses
#  2. exit if failed to link up or down
# =============================================================================
#
# ----- 1. parse ipaddress and filename -----
arg_mode=""
for arg in $*
do
  case "$arg" in
    ng*)
      unitnum="${arg#ng}"
      arg_mode=""
      ;;
    inet)
      arg_mode="local"
      ;;
    dns1)
      arg_mode="dns1"
      ;;
    dns2)
      arg_mode="dns2"
      ;;
    *)
      check=`echo $arg | $CMD_GREP -E $PATTERN_IPV4`
      if [ -z "$check" ]; then
        $arg=""
      fi
      case "$arg_mode" in
        local)
          local_ip_addr="$arg"
          arg_mode="remote"
          ;;
        remote)
          remote_ip_addr="$arg"
          arg_mode=""
          ;;
        dns1)
          pri_dns_addr="$arg"
          arg_mode=""
          ;;
        dns2)
          sec_dns_addr="$arg"
          arg_mode=""
          ;;
      esac
      ;;
  esac
done
# ----- 2. exit if failed to link up or down -----
if [ -z "$unitnum" ]; then
  echo "Failed to link up or down!"
  exit
fi


# =============================================================================
# Create the arguments for ppplnkup.sh
# -----------------------------------------------------------------------------
#  1. argument for unit number
#  2. argument for local IP addresses (and decide action)
#  3. argument for remote IP addresses
#  4. argument for DNS server addresses
# =============================================================================
#
# ----- 1. argument for unit number -----
if [ -n "$unitnum" ]; then
  arguments_string="$arguments_string -u $unitnum"
fi
# ----- 2. argument for local IP addresses (and decide action) -----
if [ -n "$local_ip_addr" ]; then
  action="LINKUP"
  arguments_string="$arguments_string -i $local_ip_addr"
  # ----- 3. argument for remote IP addresses -----
  if [ -n "$remote_ip_addr" ]; then
    arguments_string="$arguments_string -h $remote_ip_addr"
  fi
  # ----- 4. argument for DNS server addresses -----
  if [ \( -n "$pri_dns_addr" \) -o \( -n "$sec_dns_addr" \) ]; then
    arguments_string="$arguments_string -n $pri_dns_addr $sec_dns_addr"
  fi
else
  action="LINKDOWN"
fi

# =============================================================================
# Call ppplnkup.sh or ppplnkdw.sh
# =============================================================================
#
case "$action" in
  LINKUP)
    exec $PATH_PPPLNKUP $arguments_string
    ;;
  LINKDOWN)
    exec $PATH_PPPLNKDW $arguments_string
    ;;
esac
